Package: annotatr
Title: Annotation of Genomic Regions to Genomic Annotations
Version: 1.37.0
Date: 2025-06-21
Authors@R: c(
    person("Raymond G.", "Cavalcante", email = "rcavalca@umich.edu", role = c("aut", "cre")),
    person(c("Maureen A."), "Sartor", email = "sartorma@med.umich.edu", role = c("ths")))
Description: Given a set of genomic sites/regions (e.g. ChIP-seq peaks, CpGs, differentially methylated CpGs or regions, SNPs, etc.) it is often of interest to investigate the intersecting genomic annotations. Such annotations include those relating to gene models (promoters, 5'UTRs, exons, introns, and 3'UTRs), CpGs (CpG islands, CpG shores, CpG shelves), or regulatory sequences such as enhancers. The annotatr package provides an easy way to summarize and visualize the intersection of genomic sites/regions with genomic annotations.
Depends: R (>= 3.5.0)
Imports: AnnotationDbi, AnnotationHub, dplyr, GenomicFeatures (>=
        1.61.4), GenomicRanges (>= 1.61.1), Seqinfo, ggplot2 (>=
        3.5.0), IRanges, methods, readr, regioneR, reshape2, rlang,
        rtracklayer (>= 1.69.1), S4Vectors (>= 0.23.10), stats, utils
Suggests: GenomeInfoDb, BiocStyle, devtools, knitr, org.Dm.eg.db,
        org.Gg.eg.db, org.Hs.eg.db, org.Mm.eg.db, org.Rn.eg.db,
        rmarkdown, roxygen2, testthat,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene,
        TxDb.Dmelanogaster.UCSC.dm6.ensGene,
        TxDb.Drerio.UCSC.danRer10.refGene,
        TxDb.Drerio.UCSC.danRer11.refGene,
        TxDb.Ggallus.UCSC.galGal5.refGene,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene,
        TxDb.Mmusculus.UCSC.mm9.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene,
        TxDb.Mmusculus.UCSC.mm39.knownGene,
        TxDb.Rnorvegicus.UCSC.rn4.ensGene,
        TxDb.Rnorvegicus.UCSC.rn5.refGene,
        TxDb.Rnorvegicus.UCSC.rn6.refGene,
        TxDb.Rnorvegicus.UCSC.rn7.refGene
VignetteBuilder: knitr
BugReports: https://www.github.com/rcavalcante/annotatr/issues
License: GPL-3
LazyData: true
RoxygenNote: 7.3.3
Encoding: UTF-8
biocViews: Software, Annotation, GenomeAnnotation, FunctionalGenomics,
        Visualization
git_url: https://git.bioconductor.org/packages/annotatr
git_branch: devel
git_last_commit: 3e6ad83
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-07
NeedsCompilation: no
Packaged: 2025-11-07 21:14:08 UTC; biocbuild
Author: Raymond G. Cavalcante [aut, cre],
  Maureen A. Sartor [ths]
Maintainer: Raymond G. Cavalcante <rcavalca@umich.edu>
