# Generated by roxygen2: do not edit by hand

export(annotate_regions)
export(annotatr_cache)
export(build_ah_annots)
export(build_annotations)
export(builtin_annotations)
export(builtin_genomes)
export(expand_annotations)
export(plot_annotation)
export(plot_categorical)
export(plot_coannotations)
export(plot_numerical)
export(plot_numerical_coannotations)
export(randomize_regions)
export(read_annotations)
export(read_regions)
export(subset_order_tbl)
export(summarize_annotations)
export(summarize_categorical)
export(summarize_numerical)
export(tidy_annotations)
import(AnnotationDbi, except='select')
import(AnnotationHub, except='query')
import(GenomicFeatures)
import(GenomicRanges, except=c('union','setdiff','intersect','union'))
import(dplyr)
import(ggplot2)
import(methods)
importClassesFrom(S4Vectors,Hits)
importClassesFrom(S4Vectors,Rle)
importClassesFrom(Seqinfo,Seqinfo)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,splitAsList)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqnames)
importFrom(readr,read_tsv)
importFrom(regioneR,randomizeRegions)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(rtracklayer,import)
importFrom(rtracklayer,import.bed)
importFrom(stats,as.formula)
importFrom(utils,combn)
importFrom(utils,data)
