% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_txdb_name}
\alias{get_txdb_name}
\title{Function to get correct TxDb.* package name based on genome}
\usage{
get_txdb_name(genome = annotatr::builtin_genomes())
}
\arguments{
\item{genome}{A string giving the genome assembly.}
}
\value{
A string giving the name of the correct TxDb.* package name based on \code{genome}.
}
\description{
Function to get correct TxDb.* package name based on genome
}
